package com.jenny.magic.datagen;

import com.jenny.magic.blocks.blocks;
import com.jenny.magic.entities.client.ProjectileBlocks;
import net.minecraft.data.PackOutput;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.level.block.Block;
import net.minecraftforge.client.model.generators.BlockStateProvider;
import net.minecraftforge.client.model.generators.ConfiguredModel;
import net.minecraftforge.client.model.generators.ModelFile;
import net.minecraftforge.common.data.ExistingFileHelper;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.registries.RegistryObject;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;

import static com.jenny.magic.Magic.MODID;

public class ModBlockStateProvider extends BlockStateProvider {
    public ModBlockStateProvider(PackOutput output, ExistingFileHelper exFileHelper) {
        super(output, MODID, exFileHelper);
    }

    @Override
    protected void registerStatesAndModels() {
        test(blocks.ANTI_SPAWNER);
        //blockWithItem(blocks.ANTI_SPAWNER);
        simpleBlock(ProjectileBlocks.PROJECTILE_HURTFUL.get(), cubeAll(ProjectileBlocks.PROJECTILE_HURTFUL.get()));
    }

    private void test(@NotNull RegistryObject<Block> blockRegistryObject) {
        Block block = blockRegistryObject.get();
        ModelFile model = cube(block);
        this.getVariantBuilder(block).forAllStates(blockState -> ConfiguredModel.builder().modelFile(model).build());
        simpleBlockItem(block, model);
    }

    private void BlockCubeAll(@NotNull RegistryObject<Block> blockRegistryObject) {
        Block block = blockRegistryObject.get();
        this.getVariantBuilder(block).forAllStates(blockState -> ConfiguredModel.builder().modelFile(cubeAll(block)).build());
    }

    private void blockWithItem(@NotNull RegistryObject<Block> blockRegistryObject) {
        simpleBlockWithItem(blockRegistryObject.get(), cubeAll(blockRegistryObject.get()));
    }

    private void SideTop(@NotNull RegistryObject<Block> blockRegistryObject) {
        simpleBlockWithItem(blockRegistryObject.get(), topSide(blockRegistryObject.get()));
    }

    private void blockItem(@NotNull RegistryObject<Block> blockRegistryObject) {
        simpleBlockItem(blockRegistryObject.get(), new ModelFile.UncheckedModelFile(MODID +
                ":block/" + Objects.requireNonNull(ForgeRegistries.BLOCKS.getKey(blockRegistryObject.get())).getPath()));
    }

    private void topBottom2Sides(@NotNull RegistryObject<Block> blockRegistryObject) {
        simpleBlockWithItem(blockRegistryObject.get(), northEastTopBottom(blockRegistryObject.get()));
    }

    private ResourceLocation key(Block block) {
        return ForgeRegistries.BLOCKS.getKey(block);
    }

    private @NotNull String name(Block block) {
        return key(block).getPath();
    }

    private ResourceLocation extend(@NotNull ResourceLocation rl, String suffix) {
        return new ResourceLocation(rl.getNamespace(), rl.getPath() + suffix);
    }

    public void sideTopBottom(@NotNull RegistryObject<Block> blockRegistryObject) {
        Block block = blockRegistryObject.get();
        ModelFile model = models().cubeBottomTop(name(block), extend(blockTexture(block), "_side"), extend(blockTexture(block), "_bottom"), extend(blockTexture(block), "_top"));
        this.getVariantBuilder(block).forAllStates(blockState -> ConfiguredModel.builder().modelFile(model).build());
        simpleBlockItem(block, model);
    }

    public ModelFile northEastTopBottom(Block block) {
        return models().cube(name(block), extend(blockTexture(block), "_bottom"), extend(blockTexture(block), "_top"), extend(blockTexture(block), "_nw"), extend(blockTexture(block), "_se"), extend(blockTexture(block), "_se"), extend(blockTexture(block), "_nw")).texture("particle", extend(blockTexture(block), "_se"));
    }

    public ModelFile cube(Block block) {
        return models().cubeAll(name(block), blockTexture(block));
    }

    public ModelFile topSide(Block block) {
        return models().cubeColumn(name(block), extend(blockTexture(block), "_side"), extend(blockTexture(block), "_top"));
    }


}