package com.jenny.magic.entities;

import com.jenny.magic.sounds.sounds;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.util.Mth;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.Vec3;
import org.jetbrains.annotations.NotNull;

public abstract class BaseWandProjectile extends AbstractArrow {
    protected BaseWandProjectile(EntityType<? extends AbstractArrow> pEntityType, Level pLevel) {
        super(pEntityType, pLevel);
    }

    public void shootFromRotation(Entity pShooter, float pX, float pY, float pZ, float pVelocity) {
        float f = -Mth.sin(pY * ((float) Math.PI / 180F)) * Mth.cos(pX * ((float) Math.PI / 180F));
        float f1 = -Mth.sin((pX + pZ) * ((float) Math.PI / 180F));
        float f2 = Mth.cos(pY * ((float) Math.PI / 180F)) * Mth.cos(pX * ((float) Math.PI / 180F));
        Vec3 delta = new Vec3(f, f1, f2).normalize().multiply(pVelocity, pVelocity, pVelocity).add(pShooter.getDeltaMovement());
        Vec3 pos = pShooter.position().add(delta.scale(0.3).add(0, pShooter.getEyeHeight(), 0));
        this.setDeltaMovement(delta);
        this.setPos(pos);
        this.setOwner(pShooter);
    }

    public void tick() {
        if (inGroundTime > 1) {
            discard();
        }
        super.tick();
        if (level().isClientSide) {
            flyingParticles();
        }
    }

    @Override
    protected void onHitEntity(@NotNull EntityHitResult pResult) {
        if (level().isClientSide) {
            hitParticles();
        }
        hitLivingEntity(pResult);
    }

    @Override
    protected void onHitBlock(@NotNull BlockHitResult pResult) {
        if (level().isClientSide) {
            hitParticles();
        }
        hitBlock(pResult);
        super.onHitBlock(pResult);
    }

    protected Vec3 particlePos(double dist) {
        return new Vec3(
                level().getRandom().nextFloat(),
                level().getRandom().nextFloat(),
                level().getRandom().nextFloat()
        ).normalize().scale(dist * (level().getRandom().nextFloat() - 0.5f) * 2);
    }

    public void flyingParticles() {
        if (particleFlying() != null) {
            for (int i = 0; i < 5; i++) {
                Vec3 pos = particlePos(0.5).add(position());
                this.level().addParticle(particleFlying(), pos.x, pos.y, pos.z, 0, 0, 0);
            }
        }
    }

    public void hitParticles() {
        hitParticles(position());
    }

    public void hitParticles(Vec3 origin) {
        if (particleHit() != null) {
            for (int i = 0; i < 20; i++) {
                Vec3 particlePos = particlePos(0.5).add(origin);
                level().addParticle(particleHit(), particlePos.x, particlePos.y, particlePos.z, 0, 0, 0);
            }
        }
    }

    public void hitParticles(Entity entity) {
        hitParticles(entity.getBoundingBox().getCenter());
    }

    protected void hitLivingEntity(EntityHitResult pResult) {
    }

    protected void hitBlock(BlockHitResult pResult) {
    }

    @Override
    protected @NotNull SoundEvent getDefaultHitGroundSoundEvent() {
        return sounds.TEST.get();
    }

    public ParticleOptions particleFlying() {
        return null;
    }

    public ParticleOptions particleHit() {
        return null;
    }

    @Override
    protected ItemStack getPickupItem() {
        return ItemStack.EMPTY;
    }

    protected DamageSource getDamageSource(EntityHitResult entityHitResult) {
        Entity owner = this.getOwner();
        DamageSource damagesource;
        if (owner == null) {
            damagesource = this.damageSources().arrow(this, this);
        } else {
            damagesource = this.damageSources().arrow(this, owner);
            if (owner instanceof LivingEntity lOwner) {
                lOwner.setLastHurtMob(entityHitResult.getEntity());
            }
        }
        return damagesource;
    }
}
