package com.jenny.magic.items.scrolls;

import com.jenny.magic.config.ConfigServer;
import com.jenny.magic.networking.networking;
import com.jenny.magic.networking.packets.EffectS2C;
import com.jenny.magic.particles.effects;
import net.minecraft.ChatFormatting;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.Vec3;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.magic.Magic.MODID;

public class ScrollTeleport extends ScrollTeleportRandom {
    public ScrollTeleport(Properties pProperties) {
        super(pProperties);
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        use(pPlayer.position(), pPlayer, pPlayer.getItemInHand(pUsedHand), pLevel);
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    @Override
    public @NotNull InteractionResult useOn(@NotNull UseOnContext pContext) {
        use(pContext.getClickedPos().getCenter().add(0, 0.5, 0), pContext.getPlayer(), pContext.getItemInHand(), pContext.getLevel());

        return InteractionResult.SUCCESS;
    }

    public static @NotNull String dim(@NotNull Level level) {
        return level.dimension().location().toString();
    }

    static protected boolean locationSet(@NotNull CompoundTag cTag) {
        return cTag.contains("teleport_x") && cTag.contains("teleport_y") && cTag.contains("teleport_z");
    }

    protected void use(Vec3 pos, Player player, @NotNull ItemStack itemStack, Level level) {
        if (itemStack.getTag() == null) {
            itemStack.setTag(new CompoundTag());
        }
        if (locationSet(itemStack.getTag()) && !player.isCrouching()) {
            if (dimCheck(player, itemStack.getTag())) {
                if (!level.isClientSide) {
                    if (rangeCheck(player.position(), getLocation(itemStack.getTag()))) {
                        Vec3 vec = getLocation(itemStack.getTag());
                        networking.sendToPlayer(new EffectS2C(effects.EFFECT.SCROLL_TELEPORT_ORIGIN, player.position().add(0, player.getEyeHeight(), 0)), (ServerPlayer) player);
                        networking.sendToPlayer(new EffectS2C(effects.EFFECT.SCROLL_TELEPORT_TARGET, getLocation(itemStack.getTag()).add(0, player.getEyeHeight(), 0)), (ServerPlayer) player);
                        player.teleportTo(vec.x, vec.y, vec.z);
                    } else {
                        MessageDist(player);
                    }
                }
            } else {
                MessageWrongDim(player);
            }
        } else {
            itemStack.setTag(setLocation(level, player, pos, itemStack.getTag()));
            if (level.isClientSide) {
                MessageLocationSet(player);
            }
        }
    }

    private Vec3 getLocation(@NotNull CompoundTag cTag) {
        return new Vec3(
                cTag.getDouble("teleport_x"),
                cTag.getDouble("teleport_y"),
                cTag.getDouble("teleport_z")
        );
    }

    private CompoundTag setLocation(@NotNull Level level, Player player, @NotNull Vec3 pos, @NotNull CompoundTag cTag) {
        cTag.putDouble("teleport_x", pos.x);
        cTag.putDouble("teleport_y", pos.y);
        cTag.putDouble("teleport_z", pos.z);
        cTag.putString("teleport_dim", dim(level));
        return cTag;
    }

    protected boolean dimCheck(Player player, CompoundTag cTag) {
        return cTag != null && cTag.contains("teleport_dim") && cTag.getString("teleport_dim").equals(dim(player.level()));
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        if (pStack.getTag() != null && locationSet(pStack.getTag())) {
            String key = String.format("tooltip.%s.scroll_teleport.set", MODID);
            MutableComponent toolTip = Component.translatable(key);
            Vec3 pos = getLocation(pStack.getTag());
            MutableComponent toolTipPos = Component.literal((int) pos.x + ";" + (int) pos.y + ";" + (int) pos.z);
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE).append(toolTipPos.withStyle(ChatFormatting.WHITE)));
        } else {
            String key = String.format("tooltip.%s.scroll_teleport.unset", MODID);
            MutableComponent toolTip = Component.translatable(key);
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE));
        }

    }

    protected boolean rangeCheck(@NotNull Vec3 p1, Vec3 p2) {
        return ConfigServer.C_TELEPORT_SCROLL_RANGE.get() == 0 || p1.subtract(p2).length() < ConfigServer.C_TELEPORT_SCROLL_RANGE.get();
    }

    protected void MessageLocationSet(Player player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.scroll_teleport.set", MODID)));
    }

    protected void MessageWrongDim(Player player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.scroll_teleport.dim", MODID)));
    }

    protected void MessageDist(Player player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.scroll_teleport.dist", MODID)));
    }
}
