package com.jenny.magic.items.scrolls;

import com.jenny.magic.networking.networking;
import com.jenny.magic.networking.packets.EffectS2C;
import com.jenny.magic.particles.effects;
import com.mojang.datafixers.util.Pair;
import net.minecraft.ChatFormatting;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Holder;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.resources.ResourceKey;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.biome.Biome;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.magic.Magic.MODID;

public class ScrollTeleportBiome extends ScrollTeleportRandom {
    private final ResourceKey<Biome> biome;
    private final ChatFormatting color;

    public ScrollTeleportBiome(Properties pProperties, ResourceKey<Biome> biome, ChatFormatting color) {
        super(pProperties);
        this.biome = biome;
        this.color = color;
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        use(pPlayer, pPlayer.getItemInHand(pUsedHand));
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    @Override
    public @NotNull InteractionResult useOn(@NotNull UseOnContext pContext) {
        use(pContext.getPlayer(), pContext.getItemInHand());
        return InteractionResult.SUCCESS;
    }

    protected void use(@NotNull Player player, ItemStack itemStack) {
        if (!player.level().isClientSide) {
            Pair<BlockPos, Holder<Biome>> pair = ((ServerLevel) player.level()).findClosestBiome3d(biomeHolder -> biomeHolder.is(biome), player.blockPosition(), 69000, 32, 64);
            if (pair == null) {
                MessageBiomeNotFound(player);
            } else {
                networking.sendToPlayer(new EffectS2C(effects.EFFECT.SCROLL_TELEPORT_ORIGIN, player.position().add(0, player.getEyeHeight(), 0)), (ServerPlayer) player);
                networking.sendToPlayer(new EffectS2C(effects.EFFECT.SCROLL_TELEPORT_TARGET, pair.getFirst().getCenter().add(0, player.getEyeHeight(), 0)), (ServerPlayer) player);
                player.teleportTo(pair.getFirst().getX(), pair.getFirst().getY(), pair.getFirst().getZ());
            }
        }
        itemStack.shrink(1);
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        String key = String.format("tooltip.%s.scroll_teleport_biome", MODID);
        MutableComponent toolTip = Component.translatable(key);
        toolTip = toolTip.append(Component.translatable("biome." + biome.location().toLanguageKey()).withStyle(color));
        pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE));
    }

    protected void MessageBiomeNotFound(Player player) {
        player.sendSystemMessage(Component.translatable(String.format("message.%s.scroll_teleport_biome.not_found", MODID)));
    }
}
