package com.jenny.magic.items.scrolls;

import com.jenny.magic.networking.networking;
import com.jenny.magic.networking.packets.ManaAmountS2C;
import net.minecraft.ChatFormatting;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.util.RandomSource;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.Vec3;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.magic.Magic.MODID;

public abstract class ScrollTeleportRandom extends Item {

    public ScrollTeleportRandom(Properties pProperties) {
        super(pProperties);
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        String key = String.format("tooltip.%s.%s", MODID, this);
        MutableComponent toolTip = Component.translatable(key);
        if (!toolTip.getString().equals(key)) {
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE));
            super.appendHoverText(pStack, pLevel, pTooltipComponents, pIsAdvanced);
        }
    }

    public void spawnParticles(@NotNull Level level, Vec3 pos, int count, double size, ParticleOptions particle, ParticleDirection direction) {
        RandomSource rng = level.getRandom();
        Vec3 position;
        Vec3 delta;
        for (int i = 0; i < count; i++) {
            delta = new Vec3(rng.nextIntBetweenInclusive(-100, 100), rng.nextIntBetweenInclusive(-100, 100), rng.nextIntBetweenInclusive(-100, 100)).normalize().scale(size);
            if (direction == ParticleDirection.OUTWARD) {
                position = pos;
            } else {
                position = pos.subtract(delta);
                delta = delta.scale(0.02 * size);
            }
            level.addParticle(particle, position.x, position.y, position.z, delta.x, delta.y, delta.z);
        }
    }

    public enum ParticleDirection {
        INWARD,
        OUTWARD
    }

    protected void manaUpdate(ServerPlayer player) {
        networking.sendToPlayer(new ManaAmountS2C(5), player);
    }
}
