package com.jenny.magic.items.scrolls;

import com.jenny.magic.config.ConfigServer;
import com.jenny.magic.networking.networking;
import com.jenny.magic.networking.packets.EffectS2C;
import com.jenny.magic.particles.effects;
import net.minecraft.ChatFormatting;
import net.minecraft.core.BlockPos;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.util.RandomSource;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.chunk.ChunkStatus;
import net.minecraft.world.level.levelgen.Heightmap;
import net.minecraft.world.phys.Vec3;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.magic.Magic.MODID;

public class TeleportScrollRandom extends ScrollTeleportRandom {
    public TeleportScrollRandom(Properties pProperties) {
        super(pProperties);
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, @NotNull Player pPlayer, @NotNull InteractionHand pUsedHand) {
        use(pPlayer, pPlayer.getItemInHand(pUsedHand));
        return InteractionResultHolder.success(pPlayer.getItemInHand(pUsedHand));
    }

    @Override
    public @NotNull InteractionResult useOn(@NotNull UseOnContext pContext) {
        use(pContext.getPlayer(), pContext.getItemInHand());
        return InteractionResult.SUCCESS;
    }

    protected void use(@NotNull Player player, ItemStack itemStack) {
        if (!player.level().isClientSide) {
            if (itemStack.getTag() != null && itemStack.getTag().contains("teleport_x")) {
                Vec3 pos = getPos(itemStack.getTag());
                networking.sendToPlayer(new EffectS2C(effects.EFFECT.SCROLL_TELEPORT_ORIGIN, player.position().add(0, player.getEyeHeight(), 0)), (ServerPlayer) player);
                networking.sendToPlayer(new EffectS2C(effects.EFFECT.SCROLL_TELEPORT_TARGET, pos.add(0, player.getEyeHeight(), 0)), (ServerPlayer) player);
                player.teleportTo(pos.x, pos.y, pos.z);
                itemStack.shrink(1);
            } else {
                int maxDist = ConfigServer.C_TELEPORT_SCROLL_RANGE_RANDOM.get();
                RandomSource rng = player.level().getRandom();
                Vec3 randomLocation = new Vec3(
                        rng.nextIntBetweenInclusive(-maxDist, maxDist),
                        0,
                        rng.nextIntBetweenInclusive(-maxDist, maxDist)
                );
                BlockPos blockPos = new BlockPos((int) randomLocation.x, 0, (int) randomLocation.z);
                int yMax = player.level().getChunk(((int) randomLocation.x) >> 4, ((int) randomLocation.z) >> 4, ChunkStatus.FULL, true).getHeight(Heightmap.Types.WORLD_SURFACE, ((int) randomLocation.x) % 16, ((int) randomLocation.z) % 16);
                randomLocation = randomLocation.add(0, yMax + 1, 0);
                spawnParticles(player.level(), player.position().add(0, player.getEyeHeight(), 0), 20, 5, ParticleTypes.POOF, ParticleDirection.INWARD);
                spawnParticles(player.level(), randomLocation.add(0, player.getEyeHeight(), 0), 20, 0.2, ParticleTypes.END_ROD, ParticleDirection.OUTWARD);
                itemStack.setTag(setPos(player.position(), itemStack.getTag()));
                player.teleportTo(randomLocation.x, randomLocation.y, randomLocation.z);
            }
        }
    }

    protected CompoundTag setPos(@NotNull Vec3 pos, CompoundTag compoundTag) {
        compoundTag = compoundTag == null ? new CompoundTag() : compoundTag;
        compoundTag.putDouble("teleport_x", pos.x);
        compoundTag.putDouble("teleport_y", pos.y);
        compoundTag.putDouble("teleport_z", pos.z);
        return compoundTag;
    }

    protected Vec3 getPos(@NotNull CompoundTag compoundTag) {
        return new Vec3(
                compoundTag.getDouble("teleport_x"),
                compoundTag.getDouble("teleport_y"),
                compoundTag.getDouble("teleport_z")
        );
    }

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        if (pStack.getTag() != null && pStack.getTag().contains("teleport_x")) {
            String key = String.format("tooltip.%s.%s.back", MODID, this);
            MutableComponent toolTip = Component.translatable(key);
            if (!toolTip.getString().equals(key)) {
                Vec3 pos = getPos(pStack.getTag());
                MutableComponent toolTipPos = Component.literal((int) pos.x + ";" + (int) pos.y + ";" + (int) pos.z);
                pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE).append(toolTipPos.withStyle(ChatFormatting.WHITE)));
            }
        } else {
            super.appendHoverText(pStack, pLevel, pTooltipComponents, pIsAdvanced);
        }
    }
}
