package com.jenny.magic.particles;

import net.minecraft.client.Minecraft;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.util.RandomSource;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.Vec3;

import javax.annotation.Nullable;

public class effects {
    public static void executeEffect(EFFECT effect, @Nullable Vec3 origin) {
        switch (effect) {
            case WAND_VACUUM -> wandVacuum(origin);
            case SCROLL_TELEPORT_ORIGIN -> teleportOrigin(origin);
            case SCROLL_TELEPORT_TARGET -> teleportTarget(origin);
            case SCROLL_BONEMEAL -> bonemeal(origin);
            case SCROLL_HEALTH -> health(origin);
            case WAND_BOOST -> boost(origin);
        }
    }

    private static void teleportOrigin(Vec3 v) {
        for (int i = 0; i < 20; i++) {
            Vec3 randPos = randPos(1);
            Vec3 pos = v.add(randPos);
            Vec3 delta = randPos.scale(-0.1);
            level().addParticle(ParticleTypes.GLOW_SQUID_INK, pos.x, pos.y, pos.z, delta.x, delta.y, delta.z);
        }
    }

    private static void teleportTarget(Vec3 v) {
        for (int i = 0; i < 20; i++) {
            Vec3 randPos = randPosRandDist(0.2f);
            level().addParticle(ParticleTypes.GLOW_SQUID_INK, v.x, v.y, v.z, randPos.x, randPos.y, randPos.z);
        }
    }

    private static void wandVacuum(Vec3 v) {
        for (int i = 0; i < 20; i++) {
            Vec3 randPos = randPos2D(5);
            Vec3 pos = v.add(randPos);
            Vec3 delta = randPos.scale(-0.1).add(0, 0.2, 0);
            level().addParticle(ParticleTypes.GLOW_SQUID_INK, pos.x, pos.y, pos.z, delta.x, delta.y, delta.z);
        }
    }

    private static void bonemeal(Vec3 v) {
        Vec3 pos;
        for (int i = 0; i < 50; i++) {
            pos = randPosBetween2D(2, 4).add(v);
            level().addParticle(ParticleTypes.COMPOSTER, pos.x, pos.y, pos.z, 0, 0, 0);
        }
    }

    private static void health(Vec3 v) {
        Vec3 pos;
        for (int i = 0; i < 50; i++) {
            pos = randPosBetween2D(2, 4).add(v);
            level().addParticle(particles.PARTICLE_HEALTH.get(), pos.x, pos.y, pos.z, 0, 0, 0);
        }
    }

    private static void boost(Vec3 v) {
        Vec3 pos;
        for (int i = 0; i < 10; i++) {
            pos = randPos(0.5f).add(v);
            level().addParticle(ParticleTypes.POOF, pos.x, pos.y, pos.z, 0, 0, 0);
        }
    }

    public static Vec3 randPos(float maxDist) {
        return new Vec3(
                rng().nextFloat() * 2 - 1,
                rng().nextFloat() * 2 - 1,
                rng().nextFloat() * 2 - 1).normalize().scale(maxDist);
    }

    public static Vec3 randPos2D(float maxDist) {
        return new Vec3(
                rng().nextFloat() * 2 - 1,
                0,
                rng().nextFloat() * 2 - 1).normalize().scale(maxDist);
    }

    public static Vec3 randPosBetween2D(float minDist, float maxDist) {
        return randPos2D(1).scale((minDist + rng().nextFloat() * (maxDist - minDist)));
    }

    public static Vec3 randPosRandDist(float maxDist) {
        return new Vec3(
                rng().nextFloat() * 2 - 1,
                rng().nextFloat() * 2 - 1,
                rng().nextFloat() * 2 - 1).normalize().scale(rng().nextFloat() * maxDist);
    }

    private static Level level() {
        return Minecraft.getInstance().level;
    }

    private static RandomSource rng() {
        return level().getRandom();
    }

    public enum EFFECT {
        SCROLL_TELEPORT_ORIGIN,
        SCROLL_TELEPORT_TARGET,
        SCROLL_HEALTH,
        SCROLL_REPEL,
        SCROLL_WITHER,
        SCROLL_BONEMEAL,
        WAND_VACUUM,
        WAND_BOOST
    }
}
